import os
import sys
import warnings

sys.path.insert(0, os.path.abspath("../rug"))

from rug import BarChart


def test_get_ratings():
    def do_test(symbol):
        bar = BarChart(symbol)
        ratings = bar.get_ratings()

        assert 4 == len(ratings)

        for rating in ratings:
            for status, values in rating.items():
                assert type(status) is str
                assert type(values) is dict

                assert "value" in values
                assert "percent" in values

    do_test("AMD")
    do_test("TSLA")
    do_test("AAPL")


def test_get_etf_basic_info():
    def do_test(symbol):
        keys = [
            "60_month_beta",
            "alpha",
            "asset_class",
            "asset_value",
            "brand",
            "description",
            "dividend",
            "dividends",
            "dividend_yield",
            "expense_ratio",
            "first_trade_price",
            "inception",
            "index_tracked",
            "last_dividend_date",
            "latest_dividend",
            "latest_split",
            "leverage",
            "managed_assets",
            "management_fee",
            "name",
            "options",
            "pe_ratio",
            "split_date",
            "std_dev",
            "description",
        ]

        data = BarChart(symbol).get_etf_basic_info()

        for key in data.keys():
            assert key in keys

        assert 10 < len(data.keys())

    with warnings.catch_warnings():
        warnings.simplefilter("ignore")
        do_test("soxl")
        do_test("indy")
        do_test("arkb")
        do_test("TQQQ")
        do_test("vusa.l")



from .alphaquery import AlphaQuery
from .barchart import BarChart
from .finviz import FinViz
from .stockanalysis import StockAnalysis
from .stocktwits import StockTwits
from .tipranks import TipRanks


class Rug(
    AlphaQuery,
    BarChart,
    FinViz,
    StockAnalysis,
    StockTwits,
    TipRanks,
):
    """
    The Rug class provides a unified interface to all the functionality
    of the individual classes in the rug package.

    Use as:

    .. code-block:: python

        from rug import Rug

        Rug("AAPL").get_dividends()
    """

    def __init__(self, symbol=None):
        """
        Initializes the Rug object.

        :param str symbol: The stock symbol to work with.
        """
        super().__init__(symbol)



import os
import sys

sys.path.insert(0, os.path.abspath("../rug"))

import time
from datetime import date, timedelta

from rug import StockTwits


def test_get_earnings_calendar():
    api = StockTwits()
    earnings = api.get_earnings_calendar(
        date.today(), date.today() + timedelta(days=12)
    )

    assert type(earnings) is list
    assert type(earnings[0]) is dict
    assert list(earnings[0].keys()) == ["date", "symbol", "time", "when"]
    assert type(earnings[0]["date"]) is date
    assert type(earnings[0]["time"]) is time.struct_time



import html
import json
import re
import warnings

from .base import BaseAPI, HtmlTableParser
from .exceptions import DataException, SymbolNotFound


class BarChart(BaseAPI):
    def get_ratings(self):
        """
        Returns ratings for past 4 months.
        Each months is a dict with status and it's values
        where values are in absolute number (number of analyst)
        and percents (ratio).

        Returns data in format:

        .. code-block:: json

            [
                {
                    "Strong Buy":{
                        "value":"14",
                        "percent":60.86956521739131
                    },
                    "Hold":{
                        "value":"9",
                        "percent":39.130434782608695
                    }
                },
                {
                    "Strong Buy":{
                        "value":"15",
                        "percent":65.21739130434783
                    },
                    "Hold":{
                        "value":"8",
                        "percent":34.78260869565217
                    }
                },
                {
                    "Strong Buy":{
                        "value":"15",
                        "percent":62.5
                    },
                    "Moderate Buy":{
                        "value":"1",
                        "percent":4.166666666666666
                    },
                    "Hold":{
                        "value":"8",
                        "percent":33.33333333333333
                    }
                },
                {
                    "Strong Buy":{
                        "value":"17",
                        "percent":73.91304347826086
                    },
                    "Moderate Buy":{
                        "value":"1",
                        "percent":4.3478260869565215
                    },
                    "Hold":{
                        "value":"5",
                        "percent":21.73913043478261
                    }
                }
            ]

        :raises SymbolNotFound: In case the page doesn't exist/returns error code.
        :return: List of each month data.
        :rtype: list
        """

        try:
            response = self._get(
                f"https://www.barchart.com/stocks/quotes/{self.symbol.upper()}/analyst-ratings",
                use_cloudscraper=True,
            )
        except Exception as e:
            raise SymbolNotFound from e

        finds = re.findall(
            r'<analyst-rating-pie[^>]*data-content="([^"]+)"',
            response.text,
            re.DOTALL,
        )

        if finds:
            return [json.loads(html.unescape(find)) for find in finds]

        return []

    def get_etf_basic_info(self):
        """
        Fetches ETF basic data including dividends. In case any record
        cannot be fetched it's still present but has ``None`` value or
        in case of dividends an emtpy list.
        Excetpion is description record where if not found an exception
        is raised.

        Returns data in format:

            .. code-block::

                {
                    '60_month_beta': 4.08,
                    'alpha': 18.2,
                    'asset_class': 'Equity',
                    'asset_value': 60.9,
                    'brand': 'Direxion Investments',
                    'description': 'The Direxion Daily Semiconductor Bull and Bear 3X Shares seek '
                                   'daily investment results, before fees and expenses, of 300% '
                                   'or 300% of the inverse (or opposite) of the performance of '
                                   'the PHLX Semiconductor Sector Index.',
                    'dividend': 0.28,
                    'dividend_yield': 0.46,
                    'dividends': [{'amount': 0.0077, 'date': datetime.date(2010, 9, 22)},
                                  {'amount': 0.0053, 'date': datetime.date(2014, 12, 23)},
                                  {'amount': 0.121, 'date': datetime.date(2017, 12, 19)},
                                  {'amount': 0.351, 'date': datetime.date(2018, 6, 19)},
                                  {'amount': 0.452, 'date': datetime.date(2018, 9, 25)},
                                  {'amount': 0.283, 'date': datetime.date(2018, 12, 27)},
                                  {'amount': 0.243, 'date': datetime.date(2019, 3, 19)},
                                  {'amount': 0.194, 'date': datetime.date(2019, 6, 25)},
                                  {'amount': 0.313, 'date': datetime.date(2019, 9, 24)},
                                  {'amount': 0.288, 'date': datetime.date(2019, 12, 23)},
                                  {'amount': 0.079, 'date': datetime.date(2020, 3, 24)},
                                  {'amount': 0.094, 'date': datetime.date(2020, 6, 23)},
                                  {'amount': 0.003, 'date': datetime.date(2020, 12, 22)},
                                  {'amount': 0.016, 'date': datetime.date(2021, 3, 23)},
                                  {'amount': 0.014, 'date': datetime.date(2021, 12, 21)},
                                  {'amount': 0.007, 'date': datetime.date(2022, 3, 22)},
                                  {'amount': 0.019, 'date': datetime.date(2022, 6, 22)},
                                  {'amount': 0.04, 'date': datetime.date(2022, 9, 20)},
                                  {'amount': 0.038, 'date': datetime.date(2022, 12, 20)},
                                  {'amount': 0.025, 'date': datetime.date(2023, 3, 21)},
                                  {'amount': 0.038, 'date': datetime.date(2023, 6, 21)},
                                  {'amount': 0.033, 'date': datetime.date(2023, 9, 19)},
                                  {'amount': 0.063, 'date': datetime.date(2023, 12, 21)},
                                  {'amount': 0.035, 'date': datetime.date(2024, 3, 19)},
                                  {'amount': 0.149, 'date': datetime.date(2024, 6, 25)}],
                    'expense_ratio': 0.9,
                    'first_trade_price': 9.64,
                    'inception': datetime.date(2010, 3, 11),
                    'index_tracked': 'PHLX Semiconductor Sector Index',
                    'last_dividend_date': datetime.date(2024, 6, 25),
                    'latest_dividend': 0.149,
                    'latest_split': '15-1',
                    'leverage': 'Triple-Long',
                    'managed_assets': '11,981,859.20 K',
                    'management_fee': 0.9,
                    'name': 'Direxion Daily Semiconductor Bull 3X Shares',
                    'options': True,
                    'pe_ratio': 0.03,
                    'split_date': datetime.date(2021, 3, 2),
                    'std_dev': 1.31
                }

        :return: Data as a dict with all the keys.
        :rtype: dict
        """

        def get(data, label, exact=False):
            """
            Tries to fetch row from the given data based on
            literal label mathing. Uses ``in`` operator by
            default but can be switched to precise ``==`` operator
            by ``exact=True`` parameter.

            In case of multiple rows matched an exception is raised.
            In case of no row is found a warning is raised and nothing is retured.

            :param list data: List of 2 sized tuples with label and value.
            :param str label: Label of the row we want to fetch.
            :param bool exact: Uses ``in`` operator if False ``==`` otherwise.
            :return: Found row value (if any).
            """

            found = []

            for i_label, value in data:
                if not exact and label.lower() in i_label.lower():
                    found.append(value)

                elif label.lower() == i_label.lower():
                    found.append(value)

            if 1 < len(found):
                raise Exception(
                    f"Label {label} was found more than once ({len(found)} times)."
                )
            if 0 == len(found):
                warnings.warn(f"Label {label} was not found - skipping")

                return

            value = found[0]

            if isinstance(value, str):
                return value.strip()

            return value

        try:
            html = self._get(
                f"https://www.barchart.com/etfs-funds/quotes/{self.symbol.upper()}/profile",
                use_cloudscraper=True,
            )
        except Exception as e:
            raise SymbolNotFound from e

        # 1st table - overview
        finds = re.findall(r"<table>(.*?)</table>", html.text, re.DOTALL)

        if not finds:
            raise DataException("No basic data found.")

        parser = HtmlTableParser(2)
        parser.feed(finds[0])
        rows = parser.get_data()

        data = {
            "name": get(rows, "name"),
            "brand": get(rows, "fund family"),
            "inception": get(rows, "inception"),
            "index_tracked": get(rows, "underlying index"),
            "leverage": get(rows, "leverage"),
            "asset_class": get(rows, "asset class"),
        }

        # 2nd table - investment information
        parser = HtmlTableParser(2)
        parser.feed(finds[1])
        rows = parser.get_data()

        data |= {
            "alpha": get(rows, "alpha"),
            "60_month_beta": get(rows, "60-month beta"),
            "std_dev": float(dev) if (dev := get(rows, "standard deviation")) else None,
            "managed_assets": get(rows, "managed assets"),
            "asset_value": get(rows, "net asset value"),
            "first_trade_price": get(rows, "first trade"),
            "pe_ratio": get(rows, "p/e"),
            "management_fee": get(rows, "management fee"),
            "options": True if "yes" == (get(rows, "options") or "").lower() else False,
            "latest_dividend": get(rows, "latest dividend"),
            "last_dividend_date": get(rows, "last dividend date"),
            "dividend": get(rows, "annual dividend", exact=True),
            "dividend_yield": get(rows, "annual dividend yield"),
            "latest_split": get(rows, "latest split"),
            "split_date": get(rows, "split date"),
            "expense_ratio": get(rows, "expense ratio"),
        }

        # 3rd table - dividends
        if 3 == len(finds):
            parser = HtmlTableParser(2)
            parser.feed(finds[2])
            rows = parser.get_data()

            data |= {"dividends": [{"date": r[0], "amount": r[1]} for r in rows[1:]]}
        else:
            data["dividends"] = []

        # Description
        finds = re.findall(
            r"Description:(.*?)(?=<p>)<p>(.*?)</p>", html.text, re.DOTALL
        )

        if not finds:
            raise DataException("No description found.")

        data["description"] = finds[0][1]

        return data



import os
import sys

import pytest

sys.path.insert(0, os.path.abspath("../rug"))

from rug import StockAnalysis
from rug.exceptions import SymbolNotFound


def test_get_basic_info_stocks():
    def test(info):
        assert isinstance(info, dict)
        for key in [
            "company_name",
            "market",
            "description",
            "has_dividends",
            "year_low",
            "year_high",
            "pe_ratio",
            "eps",
            "market_cap",
            "60_months_beta",
            "upcoming_earnings_date",
            "similar_items",
        ]:
            assert key in info.keys()

        assert list(info["similar_items"][0].keys()) == [
            "company_name",
            "market_cap",
        ]

    test(StockAnalysis("AAPL").get_basic_info())
    test(StockAnalysis("BABA").get_basic_info())
    test(StockAnalysis("META").get_basic_info())


def test_get_basic_info_wrong_symbol():
    api = StockAnalysis("AAPLL")

    with pytest.raises(SymbolNotFound):
        api.get_basic_info()

    api = StockAnalysis("MEETA")

    with pytest.raises(SymbolNotFound):
        api.get_basic_info()



import json
import re
from datetime import datetime

from .base import BaseAPI, HtmlTableParser
from .exceptions import DataException, SymbolNotFound


class AlphaQuery(BaseAPI):
    """
    AlphaQuery.com
    """

    def get_eps(self):
        """
        Returns eps for the given ``self.symbol`` as table rows
        (list of lists) where first row is table headers for comprehension.
        Rows are sorted chronologically.

        :raises SymbolNotFound: In case the page doesn't exist/returns error code or has no data.
        :raises DataException: In case data were found but are not in valid format - only one HTML table.
        :return: List of lists with earnings.
        :rtype: list
        """

        # Get HTML.
        try:
            html = self._get(
                f"https://www.alphaquery.com/stock/{self.symbol.upper()}/earnings-history",
                use_cloudscraper=True,
            )
        except Exception as e:
            raise SymbolNotFound from e

        finds = re.findall(r"<table.*?>.*?</table>", html.text, re.DOTALL)

        # Check if the HTML contains only one table.
        if 0 == len(finds):
            raise SymbolNotFound
        if 1 < len(finds):
            raise DataException(
                "More that one table found in HTML - don't know what to do now"
            )

        parser = HtmlTableParser(columns=4)
        parser.feed(finds[0])

        return parser.get_data()

    def get_revenues(self):
        """
        Returns revenues as time went in a list of tuples
        where first is a date and the second is revenue value.

        :raises SymbolNotFound: In case the page doesn't exist/returns error code or has no data.
        :raises DataException: In case data were found but are not JSON valid.
        :return: List of EPS including dates.
        :rtype: list
        """

        # 1. fetch data.
        json_data = self._get_chart_data(
            f"https://www.alphaquery.com/stock/{self.symbol.upper()}/fundamentals/quarterly/revenue"
        )

        # 2. process data.
        if json_data:
            return list(
                map(
                    lambda i: (
                        datetime.strptime(i["x"], "%Y-%m-%dT%H:%M:%SZ").date(),
                        float(i["value"] * 10_000_000 if i["value"] else 0.0),
                    ),
                    json_data,
                )
            )

        return []

    def get_earnings(self):
        """
        Returns earnings as time went in a list of tuples
        where first is a date and the second is earning value.

        :raises SymbolNotFound: In case the page doesn't exist/returns error code or has no data.
        :raises DataException: In case data were found but are not JSON valid.
        :return: List of earnings including dates.
        :rtype: list
        """

        # 1. fetch data.
        json_data = self._get_chart_data(
            f"https://www.alphaquery.com/stock/{self.symbol.upper()}/fundamentals/quarterly/normalized-income-after-taxes",
        )

        # 2. process data.
        if json_data:
            return list(
                map(
                    lambda i: (
                        datetime.strptime(i["x"], "%Y-%m-%dT%H:%M:%SZ").date(),
                        float(i["value"] * 10_000_000 if i["value"] else 0.0),
                    ),
                    json_data,
                )
            )

        return []

    def _get_chart_data(self, url):
        """
        Digs out data from Highcharts setup under
        the given URL.

        :param url str: URL we query for the data.
        :raises SymbolNotFound: In case the page doesn't exist/returns error code or has no data.
        :raises DataException: In case data were found but are not JSON valid.
        :return: Chart data - loaded JSON object.
        :rtype: any
        """

        try:
            response = self._get(url, use_cloudscraper=True)
        except Exception as e:
            raise SymbolNotFound from e

        finds = re.findall(
            r"var chartIndicatorData = (.+?)if", response.text, re.DOTALL
        )

        if not finds:
            raise SymbolNotFound

        try:
            return json.loads(finds[0])
        except Exception as e:
            raise DataException from e



import re
from datetime import datetime

import requests

from .base import BaseAPI, HtmlTableParser
from .exceptions import DataException, HttpException, SymbolNotFound


class StockAnalysis(BaseAPI):
    def get_basic_info(self):
        """
        Downloads basic info about symbol. Data are:

        - company_name
        - market
        - description
        - has_dividends
        - year_low
        - year_high
        - pe_ratio
        - eps
        - market_cap
        - 60_months_beta
        - upcoming_earnings_date
        - similar_items
            - name
            - market_cap
        """

        def download(url):
            try:
                response = self._get(url)
                response.raise_for_status()
            except requests.HTTPError as e:
                if 404 == e.response.status_code:
                    raise SymbolNotFound
                raise HttpException from e

            return response

        def download_financials_and_others():
            def download_financials(response):
                """
                Contributes with:
                - market_cap
                - eps
                - has_dividends
                - pe_ratio
                - 60_months_beta
                - upcoming_earnings_date
                """
                # Find all tables.
                finds = re.findall(
                    r"<table[^>]*>(.*?)</table>", response.text, re.DOTALL
                )

                if not finds:
                    raise DataException(
                        f"No basic data found for symbol {self.symbol}."
                    )

                # Parse 1st table.
                table = re.findall(r"<tbody>(.*?)</tbody>", finds[0], re.DOTALL)
                first_table_rows = []

                try:
                    parser = HtmlTableParser(2)
                    parser.feed(table[0])

                    first_table_rows = parser.get_data()
                except Exception as e:
                    raise DataException(
                        f"Invalid data in table for symbol {self.symbol}."
                    ) from e

                # Parse 2nd table.
                table = re.findall(r"<tbody>(.*?)</tbody>", finds[1], re.DOTALL)
                second_table_rows = []

                try:
                    parser = HtmlTableParser(2)
                    parser.feed(table[0])

                    second_table_rows = parser.get_data()
                except Exception as e:
                    raise DataException(
                        f"Invalid data in table for symbol {self.symbol}."
                    ) from e

                return {
                    "market_cap": first_table_rows[0][1],
                    "eps": first_table_rows[3][1],
                    "has_dividends": first_table_rows[2][1] != "n/a",
                    "pe_ratio": first_table_rows[7][1],
                    "60_months_beta": second_table_rows[3][1],
                    "upcoming_earnings_date": datetime.strptime(
                        second_table_rows[5][1], "%b %d, %Y"
                    ),
                }

            def download_company_name(response):
                """
                Contributes with:
                - company_name
                """
                try:
                    h1 = re.findall(r"<h1[^>]*>(.*?)</h1>", response.text, re.DOTALL)
                    company_name = re.sub(r"\(.*?\)", "", h1[0]).strip()
                except Exception as e:
                    raise DataException(
                        f"Invalid data for company name for symobl {self.symbol}."
                    ) from e

                return {"company_name": company_name}

            def download_description(response):
                """
                Contributes with:
                - description
                """
                # For share
                try:
                    description = re.findall(
                        f"About {self.symbol.upper()}</h2>[^<]?<p>(.*?)</p>",
                        response.text,
                        re.DOTALL,
                    )[0]
                except Exception:
                    # For ETF
                    try:
                        description = re.findall(
                            f"About {self.symbol.upper()}</h2>.*?</div>[^<]<p>(.*?)</p>",
                            response.text,
                            re.DOTALL,
                        )[0]
                    except Exception as e:
                        raise DataException(
                            f"Invalid data for description for symbol {self.symbol}."
                        ) from e

                return {"description": description}

            # Try stocks data.
            response = download(
                f"https://stockanalysis.com/stocks/{self.symbol.lower()}/"
            )

            # Compile output.
            data = download_financials(response)
            data |= download_company_name(response)
            data |= download_description(response)

            return data

        def download_basics():
            response = download(
                f"https://stockanalysis.com/api/quotes/s/{self.symbol.lower()}"
            )

            try:
                data = response.json()
                data = data["data"]
            except Exception:
                raise DataException(f"Invalid JSON data for symbol {self.symbol}.")

            return {
                "market": data["ex"],
                "year_low": data["l52"],
                "year_high": data["h52"],
            }

        def similar_items():
            try:
                response = self._get(
                    f"https://stockanalysis.com/stocks/{self.symbol.lower()}/market-cap/"
                )
            except Exception:
                return {"similar_items": []}

            # Find all tables.
            finds = re.findall(r"<table[^>]*>(.*?)</table>", response.text, re.DOTALL)

            if not finds:
                raise DataException(f"No market cap found for symbol {self.symbol}.")

            # Parse 2nd table.
            finds = re.findall(r"<tbody>(.*?)</tbody>", finds[1], re.DOTALL)
            rows = []

            try:
                parser = HtmlTableParser(2)
                parser.feed(finds[0])

                rows = parser.get_data()
            except Exception as e:
                raise DataException(
                    f"Invalid data in table for symbol {self.symbol}."
                ) from e

            # Compile output.
            return {
                "similar_items": [
                    {"company_name": name, "market_cap": market_cap}
                    for name, market_cap in rows
                ]
            }

        data = download_basics()
        data |= download_financials_and_others()
        data |= similar_items()

        return data



# import os
# import sys
#
# import pytest
#
# sys.path.insert(0, os.path.abspath("../rug"))
#
# from rug import Yahoo
# from rug.exceptions import HttpException
#
#
# def test_get_current_price():
#     api = Yahoo("AAPL")
#     prices = api.get_current_price_change()
#
#     assert isinstance(prices, dict)
#     assert list(prices.keys()) == [
#         "pre_market",
#         "current_market",
#         "post_market",
#         "state",
#     ]
#     assert prices["state"] in ("pre-market", "post-market", "open")
#     assert list(prices["pre_market"].keys()) == ["change", "value"]
#     assert list(prices["pre_market"]["change"].keys()) == ["percents", "value"]
#     assert list(prices["current_market"].keys()) == ["change", "value"]
#     assert list(prices["current_market"]["change"].keys()) == ["percents", "value"]
#     assert list(prices["post_market"].keys()) == ["change", "value"]
#     assert list(prices["post_market"]["change"].keys()) == ["percents", "value"]
#
#
# def test_get_current_price_wrong_symbol():
#     api = Yahoo("AAPLL")
#
#     with pytest.raises(HttpException):
#         api.get_current_price_change()



"""
Module enumerating all possible exceptions raised
directly by Rug.
"""


class RugException(Exception):
    """
    Base exception which is extended by every other Rug exception.
    """


class HttpException(RugException):
    """
    General HTTP exception which is raised when something goes
    wrong - for example the connection was refused.
    """


class SymbolNotFound(RugException):
    """
    Specific exception indicating requested data for the given
    symbol was not found because the symbol doesn't exist.
    """


class DataException(RugException):
    """
    Generic exception for cases where scraped data from the net
    are not as we expect them.
    """



from datetime import datetime

import requests

from .base import BaseAPI
from .exceptions import HttpException, SymbolNotFound


class TipRanks(BaseAPI):
    """
    Unofficial API wrapper class for TipRanks.com.
    Unofficial means this class calls some hidden endpoints
    and provides data that official API doesn't. Also doesn't
    need an authorization.
    """

    def get_dividends(self):
        """
        Fetches symbol dividends with following fields:

        - yield
        - amount
        - ex_date
        - payment_date
        - record_date
        - growth_since

        :return: List of dividend objects.
        :rtype: list
        """

        def download(url):
            try:
                response = self._get(url)
                response.raise_for_status()
            except requests.HTTPError as e:
                if e.response.status_code in (400, 404):
                    raise SymbolNotFound
                raise HttpException from e

            return response

        try:
            response = download(
                f"https://www.tipranks.com/stocks/{self.symbol.lower()}/stock-dividends/payload.json",
            )
            data = response.json()["models"]["stocks_dividends"][0]["list"]
        except (IndexError, KeyError):
            data = None
        except SymbolNotFound:
            try:
                response = download(
                    f"https://tr-cdn.tipranks.com/assets/prod/etf/{self.symbol.lower()}/payload.json"
                )
                data = response.json()["dividends"]["history"]
            except (IndexError, KeyError):
                data = None

        dividends = []

        if data:
            for item in data:
                dividends.append(
                    {
                        "yield": float(item["yield"] or 0) * 100,
                        "amount": float(item["amount"]),
                        "ex_date": (
                            datetime.strptime(
                                item["executionDate"], "%Y-%m-%dT%H:%M:%S.000Z"
                            ).date()
                            if item["executionDate"]
                            else None
                        ),
                        "payment_date": (
                            datetime.strptime(
                                item["payDate"], "%Y-%m-%dT%H:%M:%S.000Z"
                            ).date()
                            if item["payDate"]
                            else None
                        ),
                        "record_date": (
                            datetime.strptime(
                                item["recordDate"], "%Y-%m-%dT%H:%M:%S.000Z"
                            ).date()
                            if item["recordDate"]
                            else None
                        ),
                        "growth_since": (
                            datetime.strptime(
                                item["growthSince"], "%Y-%m-%dT%H:%M:%S.000Z"
                            )
                            if item["growthSince"]
                            else None
                        ),
                    }
                )

        return dividends

    def get_current_price_change(self):
        """
        Fetches current market price inc. pre/post market
        prices/percent/value changes. Also returns current
        market state (pre-market, open, post-market).

        Fetched stucture has following fields:

        - state (pre-market, open, post-market, closed)
        - pre_market
            - change
                - percents
                - value
            - value
        - current_market
            - change
                - percents
                - value
            - value
        - post_market
            - change
                - percents
                - value
            - value

        Values are floats (if present) or 0.0.
        Returned dict looks like:

        .. code-block:: python

            {
                "state": "open",
                "pre_market": {
                    "change": {
                        "percents": -1.32476,
                        "value": -1.42001
                    },
                    "value": 105.77
                },
                "current_market": {
                    "change": {
                        "percents": -1.6046284000000002,
                        "value": -1.7200012
                    },
                    "value": 105.47
                },
                "post_market": {
                    "change": {
                        "percents": 0.0,
                        "value": 0.0
                    },
                    "value": 0.0
                }
            }

        :return: Current/Pre/Post market numbers (all are floats).
        :rtype: dict
        """

        try:
            response = self._get(
                f"https://market.tipranks.com/api/quotes/GetQuotes?app_name=tr&tickers={self.symbol.upper()}"
            )
        except requests.HTTPError as e:
            raise HttpException from e

        try:
            data = response.json()["quotes"][0]
        except IndexError:
            raise SymbolNotFound

        output = {
            "state": "closed",
            "pre_market": {
                "change": {
                    "percents": 0.0,
                    "value": 0.0,
                },
                "value": 0.0,
            },
            "current_market": {
                "change": {
                    "percents": data["changePercent"],
                    "value": data["changeAmount"],
                },
                "value": data["price"],
            },
            "post_market": {"change": {"percents": 0.0, "value": 0.0}, "value": 0.0},
        }

        if data["isPremarket"]:
            output["pre_market"]["change"]["percents"] = data["prePostMarket"][
                "changePercent"
            ]
            output["pre_market"]["change"]["value"] = data["prePostMarket"][
                "changeAmount"
            ]
            output["pre_market"]["value"] = data["prePostMarket"]["price"]
            output["state"] = "pre-market"

        elif data["isAfterMarket"]:
            output["post_market"]["change"]["percents"] = data["prePostMarket"][
                "changePercent"
            ]
            output["post_market"]["change"]["value"] = data["prePostMarket"][
                "changeAmount"
            ]
            output["post_market"]["value"] = data["prePostMarket"]["price"]
            output["state"] = "post-market"

        elif data["isMarketOpen"]:
            output["state"] = "open"

        return output



from datetime import date

from rug import AlphaQuery


def test_get_eps():
    def do_test(symbol):
        aq = AlphaQuery(symbol)
        earnings = aq.get_eps()

        assert len(earnings) > 10

        for heading in earnings[0]:
            assert type(heading) is str

        for row in earnings[1:]:
            assert type(row[0]) is date
            assert type(row[1]) is date
            assert type(row[2]) is float
            assert type(row[3]) is float

    do_test("AMD")
    do_test("META")
    do_test("TSLA")
    do_test("NIO")


def test_get_revenues():
    def do_test(symbol):
        aq = AlphaQuery(symbol)
        revenues = aq.get_revenues()

        assert 10 <= len(revenues)

        for revenue in revenues:
            assert type(revenue[0]) is date
            assert type(revenue[1]) is float

    do_test("AMD")
    do_test("META")
    do_test("TSLA")
    do_test("NIO")


def test_get_earnings():
    def do_test(symbol):
        aq = AlphaQuery(symbol)
        earnings = aq.get_earnings()

        assert 10 <= len(earnings)

        for earning in earnings:
            assert type(earning[0]) is date
            assert type(earning[1]) is float

    do_test("AMD")
    do_test("META")
    do_test("TSLA")
    do_test("NIO")



# import pytest
#
# from rug import EtfDb
# from rug.exceptions import SymbolNotFound
#
#
# def test_get_basic_info():
#     def do_test(symbol):
#         data = EtfDb(symbol).get_basic_info()
#
#         for key in data.keys():
#             assert key in [
#                 "Issuer",
#                 "Brand",
#                 "Structure",
#                 "Expense Ratio",
#                 "Inception",
#                 "Index Tracked",
#                 "Category",
#                 "Leveraged",
#                 "Asset Class",
#                 "Asset Class Size",
#                 "Region (General)",
#                 "Region (Specific)",
#                 "Description",
#                 "Assets Under Management",
#                 "Year Low",
#                 "Year High",
#             ]
#
#     do_test("SPLG")
#     do_test("SPY")
#     do_test("TQQQ")
#
#     with pytest.raises(SymbolNotFound):
#         do_test("abcd")
#
#
# def test_get_holdings():
#     holdings = EtfDb("SPLG").get_holdings()
#     assert 15 == len(holdings)
#     assert "AAPL" in [h[0] for h in holdings]
#
#
# def test_get_etfs_by_item():
#     assert 25 == len(EtfDb("AAPL").get_etfs_by_item())
#
#     msft = EtfDb("MSFT").get_etfs_by_item()
#     assert 25 == len(msft)
#     assert str == type(msft[0][0])
#     assert str == type(msft[0][1])
#     assert str == type(msft[0][2])
#     assert float == type(msft[0][3])
#     assert float == type(msft[0][4])
#
#     with pytest.raises(SymbolNotFound):
#         EtfDb("MSFTTT").get_etfs_by_item()



import datetime
import os
import sys

import pytest

sys.path.insert(0, os.path.abspath("../rug"))

from rug import TipRanks
from rug.exceptions import SymbolNotFound


def test_get_dividends():
    # Dividends
    api = TipRanks("AAPL")
    dividends = api.get_dividends()

    assert isinstance(dividends, list)
    assert list(dividends[0].keys()) == [
        "yield",
        "amount",
        "ex_date",
        "payment_date",
        "record_date",
        "growth_since",
    ]

    if dividends[0]["ex_date"]:
        assert isinstance(dividends[0]["ex_date"], datetime.date)

    if dividends[0]["payment_date"]:
        assert isinstance(dividends[0]["payment_date"], datetime.date)

    if dividends[0]["record_date"]:
        assert isinstance(dividends[0]["record_date"], datetime.date)

    if dividends[0]["growth_since"]:
        assert isinstance(dividends[0]["growth_since"], datetime.date)

    # No dividends.
    api = TipRanks("tsla")
    dividends = api.get_dividends()

    assert isinstance(dividends, list)
    assert dividends == []


def test_get_dividends_wrong_symbol():
    api = TipRanks("AAPLL")

    with pytest.raises(SymbolNotFound):
        api.get_dividends()


def test_get_dividends_no_dividends():
    api = TipRanks("XYZ")
    assert api.get_dividends() == []

    api = TipRanks("SOXL")
    assert api.get_dividends() == []



import datetime

from rug import FinViz


def test_get_price_ratings():
    def do_test(symbol):
        fv = FinViz(symbol)
        ratings = fv.get_price_ratings()

        assert 0 < len(ratings)
        assert 5 == len(ratings[0])
        assert type(ratings[0][0]) is datetime.date
        assert type(ratings[0][1]) is str
        assert type(ratings[0][2]) is str
        assert type(ratings[0][3]) is str
        assert type(ratings[0][4]) in (str, float)

    do_test("AMD")
    do_test("META")
    do_test("TSLA")
    do_test("NIO")


def test_get_insider_trading():
    def do_test(symbol):
        fv = FinViz(symbol)
        trading = fv.get_insider_trading()

        assert 0 < len(trading)

        for row in trading:
            assert isinstance(row["person"], str)
            assert isinstance(row["relationship"], str)
            assert isinstance(row["date"], datetime.date)
            assert isinstance(row["transaction"], str)
            assert isinstance(row["price"], float)
            assert isinstance(row["amount"], int)

    do_test("AMD")
    do_test("TSLA")
    do_test("META")


def test_etf_holdings():
    def do_test(symbol, none_symbol=False):
        bar = FinViz(symbol)
        holdings = bar.get_etf_holdings()

        assert 10 == len(holdings)

        for holding in holdings:
            assert type(holding["name"]) is str

            if none_symbol:
                assert holding["symbol"] is None
            else:
                assert type(holding["symbol"]) is str

            assert type(holding["weight"]) is float

    do_test("SPLG")
    do_test("QQQ")
    do_test("TQQQ", True)
